import { $ } from 'zx';
import * as webPush from 'web-push';
import Yaml from 'yaml';
import {
  parseYaml,
  getRandomString,
  getCWDPath,
  readCWDFile,
  moveCWDFile,
  writeCWDFile,
  removeCWDFile,
} from './utilities';
import { logDebug } from './log';

type Options = {
  verbose?: boolean;
};

export async function generateFiles(
  chirpyDomain: string,
  hasuraDomain: string,
  { verbose }: Options,
): Promise<void> {
  // Download the repo
  await $`curl -L https://github.com/devrsi0n/chirpy/archive/main.tar.gz | tar -xz`;

  await moveCWDFile('./chirpy-main/services/hasura', './hasura');
  await moveCWDFile('./chirpy-main/services/chirpy', './chirpy');
  await removeCWDFile('./chirpy-main');
  const chirpyDCFile = await readCWDFile('./chirpy/docker-compose.tmpl.yml');
  logDebug(verbose, 'Chirpy docker compose file: ', chirpyDCFile);

  const chirpyURL = `https://${chirpyDomain}`;
  // We must use `\n` to replace `${}`, or it can't work
  chirpyDCFile.replace('${NEXT_PUBLIC_APP_URL}\n', `${chirpyURL}\n`);
  chirpyDCFile.replace('${NEXTAUTH_URL}\n', `${chirpyURL}\n`);
  chirpyDCFile.replaceAll(
    '${NEXT_PUBLIC_HASURA_HTTP_ORIGIN}\n',
    `https://${hasuraDomain}:8000/v1/graphql\n`,
  );
  chirpyDCFile.replaceAll(
    '${NEXT_PUBLIC_HASURA_WS_ORIGIN}\n',
    `wss://${hasuraDomain}:8000/v1/graphql\n`,
  );

  // Setup hasura admin secret
  const hasuraAdminSecret = getRandomString(32);
  chirpyDCFile.replaceAll(
    '${HASURA_GRAPHQL_ADMIN_SECRET}\n',
    `${hasuraAdminSecret}\n`,
  );
  chirpyDCFile.replaceAll('${HASURA_ADMIN_SECRET}\n', `${hasuraAdminSecret}\n`);
  const hasuraConfig = await parseYaml(getCWDPath('./hasura/config.yaml'));
  hasuraConfig.admin_secret = hasuraAdminSecret;
  hasuraConfig.endpoint = `https://${hasuraDomain}:8000`;
  await writeCWDFile('./hasura/config.yaml', Yaml.stringify(hasuraConfig));

  const hasuraDCFile = await readCWDFile('./hasura/docker-compose.tmpl.yml');
  // Setup hasura jwt secret
  const hasuraJwtSecret = getRandomString(129);
  hasuraDCFile.replaceAll(
    '${HASURA_GRAPHQL_JWT_SECRET}\n',
    `${hasuraJwtSecret}\n`,
  );
  chirpyDCFile.replaceAll('${NEXTAUTH_SECRET}\n', `${hasuraJwtSecret}\n`);

  // Setup hasura event secret
  const hasuraEventSecret = getRandomString(32);
  chirpyDCFile.replaceAll('${HASURA_EVENT_SECRET}\n', `${hasuraEventSecret}\n`);
  hasuraDCFile.replaceAll('${HASURA_EVENT_SECRET}\n', `${hasuraEventSecret}`);

  // Setup web-push keys
  const vapidKeys = webPush.generateVAPIDKeys();
  chirpyDCFile.replaceAll('${NEXT_PUBLIC_VAPID}\n', `${vapidKeys.publicKey}\n`);
  chirpyDCFile.replaceAll('${PRIVATE_VAPID}', `{vapidKeys.privateKey}\n`);

  // Setup Caddy
  let chirpyCaddy = await readCWDFile('./chirpy/Caddyfile.tmpl');
  chirpyCaddy = chirpyCaddy.replaceAll('<your-domain>', chirpyDomain);
  await writeCWDFile('./chirpy/Caddyfile', chirpyCaddy);
  logDebug(verbose, 'Chirpy docker compose content', chirpyDCFile);

  await writeCWDFile('./chirpy/docker-compose.yml', chirpyDCFile);

  let hasuraCaddy = await readCWDFile('./hasura/Caddyfile.tmpl');
  hasuraCaddy = hasuraCaddy.replaceAll('<your-domain>', hasuraDomain);
  await writeCWDFile('./hasura/Caddyfile', hasuraCaddy);

  logDebug(verbose, 'Hasura docker compose content', hasuraDCFile);

  await writeCWDFile('./hasura/docker-compose.yml', hasuraDCFile);
}
